# app_delegate.rb
# RCRunner
#
# Created by Juri Pakaste on 20.11.10.
# Copyright 2010 Juri Pakaste. All rights reserved.

require "tests_view_controller"
require "tests_provider_facade"
require "minitest_provider"

module RCRunner
  class AppDelegate
    attr_writer :window

    def load_test_modules
      errfiles = {}
      @allfiles.each do |file|
        begin
          load(file)
        rescue Exception => e
          errfiles[file] = e
        end
      end

      errparts = []
      errfiles.each do |file, exc|
        errparts << "#{file}: #{exc.message}"
      end

      if errparts.length > 0
        alert = NSAlert.alloc.init
        alert.addButtonWithTitle("Close")
        alert.setMessageText("Errors while loading test modules")
        alert.setInformativeText("Some of the files you specified weren't loaded successfully.\n\n" + errparts.join("\n"))
        alert.setAlertStyle(NSWarningAlertStyle)
        alert.beginSheetModalForWindow(@window, modalDelegate:nil, didEndSelector:nil, contextInfo:nil)
      end

      tprovider = TestsProviderFacade.new
      tprovider.tests_providers = [MiniTestProvider.new]
      @tests_view_controller.tests_provider = tprovider

      puts "tests: #{tprovider.tests}"
    end

    def create_window_name
      dirparts = Dir.pwd.split("/").reverse
      incparts = dirparts.take_while { |d| ["Debug", "Release", "build"].include? d }
      if dirparts.length > incparts.length
        incparts << dirparts[incparts.length]
      end

      incparts.reverse.join("/")
    end
    
    def applicationDidFinishLaunching(notification)
      @tests_view_controller = TestsViewController.alloc.initWithNibName('TestsView', bundle:nil)
      @window.title = self.create_window_name
      @window.setRepresentedFilename(Dir.pwd)
      @window.setFrameAutosaveName(@window.representedFilename)
      @window.setContentView(@tests_view_controller.view)

      @window.makeKeyAndOrderFront(self)

      @allfiles = []
      autorun = false
      NSProcessInfo.processInfo.arguments.drop(1).each do |arg|
        if arg == '--autorun'
          autorun = true
          next
        end
        argfiles = Dir.glob(arg)
        @allfiles += argfiles
      end
      @allfiles = @allfiles.uniq
      
      puts "all files: #{@allfiles}"

      self.load_test_modules

      if autorun
        puts "calling run_all"
        @tests_view_controller.run_all
      end
    end

    def reload_menu_clicked(sender)
      self.load_test_modules
    end
    
    # delegate run* menu items to tests_view_controller
    
    def run_all_menu_clicked(sender)
      @tests_view_controller.run_clicked(sender)
    end

    def run_selected_menu_clicked(sender)
      @tests_view_controller.run_selected_clicked(sender)
    end

    def run_failed_menu_clicked(sender)
      @tests_view_controller.run_failed_clicked(sender)
    end
  end
end
